#!/usr/bin/env python3

"""
Testing tool for Sophos Intercept X Advanced with EDR
to aid with product demonstration and evaluations.
"""

__author__ = "gregiddon"
__version__ = "1.0.2"
__license__ = "Apache-2.0"

# Imports
import os
import sys
import argparse
import logging
import requests
import subprocess
import zipfile
import urlparse
from gooey import Gooey

START         = '=====START====='
END           = '======END======'
UNKNOWN_PE    = 'http://sophostest.com/reputation/unknown.exe'
LOW_PE        = 'http://sophostest.com/reputation/low.exe'
MEDIUM_PE     = 'http://sophostest.com/reputation/medium.exe'
HIGH_PE       = 'http://sophostest.com/reputation/high.exe'
HIPS_PE       = 'http://sophostest.com/sophos_hips_test/sophos_hips_test.exe'
HIPS_ZIP      = 'http://sophostest.com/sophos_hips_test/sophos_hips_test.zip'
HIGHSCORE_PE  = 'https://drive.google.com/uc?id=14sX8kPo5qBt6O5NJxN7wOo184C6ZvlNf&export=download'
EICAR         = 'https://secure.eicar.org/eicar.com'
MALWARE_URL   = 'http://sophostest.com/malware/index.html'
CRIMINAL_URL  = 'http://sophostest.com/criminal-activity/index.html'
DOWNLOADS_URL = 'http://sophostest.com/downloads/index.html'
C2_URL        = 'http://sophostest.com/callhome/index.html'
READ_EULA     = """
END USER LICENSE AGREEMENT & PRIVACY POLICY

Use of this tool is subject to the Sophos End User License Agreement (EULA).
https://www.sophos.com/en-us/legal/sophos-end-user-license-agreement.aspx

You must accept the EULA to continue, so please read it carefully.

You also acknowledge that Sophos processes personal data in accordance with
the Sophos Privacy Policy.
https://www.sophos.com/en-us/legal/sophos-group-privacy-policy.aspx

Run this tool again and check the "accept_eula" checkbox to accept the
Sophos End User License Agreement and acknowledge the Sophos Privacy Policy.
"""
CURDIR        = os.getcwd()

class Fakedrop:
    temp_files = []
    arguments = []

    def __init__(self, arguments):
        print(START)
        self.arguments = arguments
        if self.arguments.accept_eula:
            if self.arguments.discovery:
                self.discovery()
            if self.arguments.trigger:
                self.trigger()
        else:
            print(READ_EULA)

    def __del__(self):
        #self._delete_temp_files()
        print(END)

    def discovery(self):
        logging.debug('mode enabled')
        self._iterate_documents(os.curdir)
        self._get_and_execute(UNKNOWN_PE)
        self._get_and_execute(LOW_PE)
        if self.arguments.skip_web is False:
            self._get_website(CRIMINAL_URL)
            self._get_website(DOWNLOADS_URL)
        else:
            print('skipping website calls')
        logging.debug('done')

    def trigger(self):
        logging.debug('mode enabled')
        self.discovery()
        self._get_and_execute(HIGHSCORE_PE)
        if self.arguments.skip_hips is False:
            self._get_and_execute(HIPS_PE)
        else:
            print('skipping HIPS test')
        self._get_and_execute(EICAR)
        if self.arguments.skip_web is False:
            self._get_website(C2_URL)
            self._get_website(MALWARE_URL)
        else:
            print('skipping website calls')
        logging.debug('done')

    def _get_and_execute(self, url):
        f = None
        try:
            f = self._download_file(url)
        except:
            print('unable to download ' + url)
            return False
        else:
            if any(".zip" in s for s in f):
                try:
                    f = self._unzip_file(f)
                except:
                    print('unable to extract ' + f)
                    return False
            else:
                try:
                    f = self._execute_file(f)
                except:
                    print('unable to execute ' + f)
                    return False
                else:
                    return True

    def _delete_temp_files(self):
        for file_path in self.temp_files:
            logging.debug(file_path)
            os.remove(file_path)
            print('deleted ' + self._get_filename(file_path))

    def _iterate_documents(self, directory):
        logging.debug('reading ' + directory)
        for file_name in os.listdir(directory):
            file_path = directory + '\\' + file_name
            if os.path.isfile(file_path):
                try:
                    with open(file_path, 'r') as open_file:
                        print('read file ' + file_name)
                except:
                    print('unable to read file ' + file_name)
        logging.debug('done')

    def _execute_file(self, file_paths):
        for file_path in file_paths:
            file_name = os.path.basename(file_path)
            print('executing ' + file_name)
            try:
                subprocess.Popen(file_path, shell=False, stdout=None)
            except:
                print('unable to execute ' + file_name)
                return False
            else:
                print('executed ' + file_name)
                return True

    def _download_file(self, url):
        logging.debug(url)
        if self._validate_url(url):
            file_http = requests.get(url, stream=True)
            file_http = file_http.content
            file_name = self._get_filename(url)
            logging.info(file_name)
            try:
                with open(file_name, 'wb') as file_disk:
                    file_disk.write(file_http)
            except:
                logging.warning('unable to write ' + file_name)
                print('unable to write ' + file_name)
                return False
            else:
                file_path = CURDIR + '\\' + file_name
                print('downloaded ' + file_name)
                self.temp_files.append(file_path)
                return [file_path]
        else:
            logging.debug('unable to download ' + url)
            return False

    def _unzip_file(self, file_paths):
        for file_path in file_paths:
            logging.debug(file_path)
            with zipfile.ZipFile(file_path, 'r') as zip_file:
                extracted = []
                for name in zip_file.namelist():
                    file_name = os.path.basename(name)
                    if file_name:
                        zipped = zip_file.read(name)
                        with open(file_name, 'wb') as file_disk:
                            file_disk.write(zipped)
                        extracted.append(CURDIR + '\\' + file_name)
                        self.temp_files.append(CURDIR + '\\' + file_name)
                        print('unzipped ' + file_name)
                logging.debug('done')
                return extracted

    def _get_website(self, url):
        if self._validate_url(url):
            print('calling website ' + url)
            try:
                website = requests.get(url)
            except:
                print('unable to reach ' + url)
                return False
            else:
                if website.status_code == 200:
                    print('successfully reached ' + url)
                elif website.status_code == 403:
                    print('something blocked ' + url)
                return True

    def _validate_url(self, url):
        logging.debug('checking URL: ' + str(url))
        try:
            result = urlparse.urlparse(url)
            if result.hostname == None:
                logging.warning('invalid URL: ' + str(url))
                return False
            else:
                logging.debug('valid URL: ' + str(url))
                return True
        except:
            logging.warning('invalid URL: ' + str(url))
            return False

    def _get_filename(self, url):
        filename = os.path.basename(url)
        if filename == 'uc?id=14sX8kPo5qBt6O5NJxN7wOo184C6ZvlNf&export=download':
            return 'highscore.zip'
        return filename

class Unbuffered(object):
   def __init__(self, stream):
       self.stream = stream
   def write(self, data):
       self.stream.write(data)
       self.stream.flush()
   def writelines(self, datas):
       self.stream.writelines(datas)
       self.stream.flush()
   def __getattr__(self, attr):
       return getattr(self.stream, attr)

def logger(args):
    log_adjust = max(min(args.quiet - args.verbose, 2), -2) * 10
    log_file = ''
    if args.logfile:
        log_file = args.logfile
    logging.basicConfig(
        filename=log_file,
        level=logging.WARNING + log_adjust,
        format='%(levelname)-8s %(module) 10s: %(funcName)s %(message)s')
    if args.logfile:
        logger = logging.getLogger()
        handler = logging.FileHandler(log_file)
        logger.addHandler(handler)
    logging.info('verbosity increased')
    logging.debug('verbosity increased')

@Gooey(
    program_name='Sophos EDR Demo Tool v'+__version__,
    advanced=True,
    navigation='TABBED',
    show_sidebar=False,
    tabbed_groups=True)
def main():
    sys.stdout = Unbuffered(sys.stdout)
    parser = argparse.ArgumentParser(
        prog='Fakedrop',
        description=__doc__,
        usage='%(prog)s [options]')
    required = parser.add_argument_group(
        'Required',
        description="""
        Use of this tool is subject to the Sophos End User License Agreement (EULA).
        https://www.sophos.com/en-us/legal/sophos-end-user-license-agreement.aspx

        You must accept the EULA to continue, so please read it carefully.

        You also acknowledge that Sophos processes personal data in accordance with the Sophos Privacy Policy.
        https://www.sophos.com/en-us/legal/sophos-group-privacy-policy.aspx
        """)
    required.add_argument(
        '--accept-eula',
        action='store_true',
        default=False,
        help="I accept the Sophos End User License Agreement and acknowledge the Sophos Privacy Policy.")
    mode_select = parser.add_argument_group('Mode Selection (select one)')
    mode = mode_select.add_mutually_exclusive_group()
    mode.add_argument(
        '-d', '--discovery',
        action='store_true',
        default=True,
        help='drop artefacts without throwing a malicious detection')
    mode.add_argument(
        '-t', '--trigger',
        action='store_true',
        help='drop artefacts and generate a malicious detection')
    optional = parser.add_argument_group('Optional')
    optional.add_argument(
        '--skip-hips',
        action='store_true',
        default=False,
        help='skip HIPS test file')
    optional.add_argument(
        '--skip-web',
        action='store_true',
        default=False,
        help='skip calls to suspicious URLs')
    optional.add_argument(
        '-v', '--verbose',
        action='count',
        default=0,
        help='increase logging output')
    optional.add_argument(
        '-q', '--quiet',
        action='count',
        default=0,
        help='decrease logging output')
    optional.add_argument(
        '-l', '--logfile',
        metavar='FILE',
        help='write logging events to file instead of the terminal')
    args = parser.parse_args()
    logger(args)
    edr = Fakedrop(args)

if __name__ == "__main__":
    main()